*================= Program: PSI_AREA_OSERVED.SAS ==================;
*
*  DESCRIPTION:
*         Uses Proc Summary to calculate observed rates for
*         patient safety indicators across stratifiers.
*         Output stratified by AREA, AGECAT, SEXCAT and RACECAT from 
*         the population file and PSI_ALL_MEASURES output.
*         Variables created by this program are PAPSxx and OAPSxx.
*
*              >>> AHRQ QI ICD-09 VERSION 6.0.2 <<<
*                 >>> VERSION DATE JULY 2017<<<
*===================================================================;
* See the PSI_ICD09_v6_Release_Notes.txt file for version change details.;

*PATHNAME specifies the location of the PSI folder which includes the
          Programs, Macros, and SASdata subfolders;
%LET PATHNAME=C:\PATHNAME\PSI;                           *<===USER MUST modify;
filename CONTROL "&PATHNAME.\Programs\PSI_ALL_CONTROL.SAS";  
%INCLUDE CONTROL;


 %MACRO CTY2MA;
    %IF &MALEVL EQ 0 %THEN %DO;
        attrib MAREA length=$5
          label='FIPS STATE COUNTY CODE';
        MAREA = FIPSTCO;
    %END;
    %ELSE %IF &MALEVL EQ 1 %THEN %DO;
        attrib MAREA length=$5
          label='MODIFIED FIPS';
        MAREA = input(put(FIPSTCO,$M1AREA.),$5.);
    %END;
    %ELSE %IF &MALEVL EQ 2 %THEN %DO;
        attrib MAREA length=$5
          label='OMB 1999 METRO AREA';
        MAREA = input(put(FIPSTCO,$M2AREA.),$5.);
    %END;
    %ELSE %IF &MALEVL EQ 3 %THEN %DO;
        attrib MAREA length=$5
          label='OMB 2003 METRO AREA';
        MAREA = input(put(FIPSTCO,$M3AREA.),$5.);
    %END;
 %MEND;

 title2 'PROGRAM PSI_AREA_OBSERVED';
 title3 'AHRQ PATIENT SAFETY INDICATORS: CALCULATE ADJUSTED AREA RATES';

 * ---------------------------------------------------------------- ;
 * --- ADD POPULATION DENOMINATOR --------------------------------- ;
 * --- THIS STEP DETERMINES WHICH AREAS ARE INCLUDED IN THE     --- ;
 * --- OUTPUT BASED ON AREAS IN PSI_ALL_MEASURES.SAS            --- ;
 * ---------------------------------------------------------------- ;

 data TEMP0;
  set OUTMSR.&OUTFILE_MEAS.;

  %CTY2MA

 run;
/* IDENTIFY MAREA VALUES IN PSI_ALL_MEASURES OUTPUT. */
 proc  Sort data=TEMP0 out=MAREA(keep=MAREA) nodupkey;
   by MAREA;
 run;
/* LOAD POPULATION FOR MAREA LEVEL AND YEAR IN PSI_ALL_CONTROL. */
 data QIPOP0;
    length FIPSTCO $5 SEXCAT POPCAT AGECAT RACECAT 3 
           POP_1995 POP_1996 POP_1997 POP_1998 POP_1999
           POP_2000 POP_2001 POP_2002 POP_2003 POP_2004
           POP_2005 POP_2006 POP_2007 POP_2008 POP_2009 
           POP_2010 POP_2011 POP_2012 POP_2013 POP_2014
           POP_2015 POP 8;

    infile POPFILE missover firstobs=2;

    input FIPSTCO SEXCAT POPCAT RACECAT 
          POP_1995 POP_1996 POP_1997 POP_1998 POP_1999
          POP_2000 POP_2001 POP_2002 POP_2003 POP_2004
          POP_2005 POP_2006 POP_2007 POP_2008 POP_2009
          POP_2010 POP_2011 POP_2012 POP_2013 POP_2014
          POP_2015;

    %CTY2MA

    if POPCAT in (1,2,3,4)            then AGECAT = 0;
    else if POPCAT in (5,6,7,8)       then AGECAT = 1;
    else if POPCAT in (9,10,11,12,13) then AGECAT = 2;
    else if POPCAT in (14,15)         then AGECAT = 3;
    else                                   AGECAT = 4;

    POP = POP_&POPYEAR.;

 run;

 proc Summary data=QIPOP0 nway;
   class MAREA POPCAT AGECAT SEXCAT RACECAT;
   var POP;
   output out=QIPOP sum=;
 run;

 proc Sort data=QIPOP;
   by MAREA POPCAT AGECAT SEXCAT RACECAT;
 run;
/* LIMIT POPULATION TOTALS TO MAREA CODES FOUND IN PSI_ALL_MEASURES OUTPUT. */
 data QIPOP(keep=MAREA POPCAT AGECAT SEXCAT RACECAT POP);
   merge  MAREA(in=X) QIPOP(in=Y);
   by     MAREA;

   if X and Y;

 run;

 * --------------------------------------------------------------- ;
 * --- PATIENT SAFETY INDICATOR ADJUSTED RATES ------------------- ;
 * --------------------------------------------------------------- ;
 * AREA-LEVEL INDICATOR DENOMINATORS ARE ADJUSTED BASED ON THE     *
 * COMBINATION OF COUNTY, AGE, SEX, AND RACE IN THE NUMERATOR.     *
 * THIS STEP ITERATES THROUGH THE OUTPUT FROM PSI_ALL_MEASURES FOR *
 * EACH MEASURE AND REDUCES THE POPULATION SUM BY THE NUMERATOR    *
 * TOTAL. THE MOD3 MACRO ADJUSTS THE RESIDENTIAL POPULATION TOTAL  *
 * BASED ON THE MEASURE RELAVENT POPULATION. ONLY VALID AREA       *
 * CODES ARE RETURNED.                                             *
 * --- N - AREA MEASURE NUMBER                                 --- *
 * --- PS - THE PATIENT SAFETY INDICATOR NAME WITHOUT THE      --- *
 * ---      PREFIX (A)                                         --- *
 * --------------------------------------------------------------- ;

 %MACRO MOD3(N,PS);

 /* CREATE TEMP TABLE WITH ALL DISCHARGES IN NUMERATOR FOR INDICATOR N. */

 data TEMP_2;
   set  OUTMSR.&OUTFILE_MEAS.(keep=KEY FIPSTCO T&PS. POPCAT AGECAT SEXCAT RACECAT);
   if T&PS. in (1);

   %CTY2MA

 run;

  /* SUM THE NUMERATOR 'T' FLAGS BY MAREA POPCAT AGECAT SEXCAT RACECAT. */
 proc Summary data=TEMP_2 NWAY;
   class  MAREA POPCAT AGECAT SEXCAT RACECAT;
   var    T&PS.;
   output out=TEMP_3 N=TCOUNT;
 run;

 proc Sort data=TEMP_3;
   by  MAREA POPCAT AGECAT SEXCAT RACECAT;
 run;

/* REDUCE THE DENOMINATOR POPULATION BY THE SUM OF THE NUMERATOR COUNT. */

 data   TEMP_4(DROP=TCOUNT);
 merge  QIPOP(IN=X keep=MAREA POPCAT AGECAT SEXCAT RACECAT POP) 
        TEMP_3(keep=MAREA POPCAT AGECAT SEXCAT RACECAT TCOUNT);
 by     MAREA POPCAT AGECAT SEXCAT RACECAT;

 if X;

 if TCOUNT > 0 then PCOUNT = POP - TCOUNT;
 else PCOUNT = POP;

 if PCOUNT < 0 then PCOUNT = 0;

 N = &N.;

 if AGECAT in (0) then PCOUNT = 0;

 if PCOUNT = 0 then delete;

 run;

/* FOR NUMERATOR, RETAIN ONLY RECORDS WITH A VALID MAREA CODE. */

 data   TEMP_3(DROP=POP);
   merge  TEMP_3(IN=X keep=MAREA POPCAT AGECAT SEXCAT RACECAT TCOUNT)
          QIPOP(keep=MAREA POPCAT AGECAT SEXCAT RACECAT POP);
   by MAREA POPCAT AGECAT SEXCAT RACECAT;
   if X;

 if POP < 0 then PCOUNT = 0;
 else if TCOUNT > 0 then PCOUNT = TCOUNT;
 else PCOUNT = 0;

 if PCOUNT = 0 then delete;

 run;

 /* COMBINE THE NUMERATOR AND DENOMINATOR */;

 data TEMP1;
 set  TEMP_3(IN=X) TEMP_4;
    if X then T&PS. = 1;
    else T&PS. = 0;
 run;

 data TEMP1Y;
   set  TEMP1;

   ONE = 1;

 run;

 * -------------------------------------------------------------- ;
 * --- AGGREGATE POPULATION COUNTS BY STRATIFIERS             --- ;
 * -------------------------------------------------------------- ;

 proc Summary data=TEMP1Y;
   class MAREA AGECAT SEXCAT RACECAT;
   var T&PS. ONE;
   output out=ADJ_&PS. SUM(T&PS. ONE)=T&PS. P&PS.;
   weight PCOUNT;
 run;

 data   ADJ_&PS.;
   set    ADJ_&PS.;
   if _TYPE_ &TYPELVLA;
   keep MAREA AGECAT SEXCAT RACECAT T&PS. P&PS. _TYPE_;
 run;

 proc Sort data = ADJ_&PS.;
   by MAREA AGECAT SEXCAT RACECAT;
 run;  quit;

 proc DATASETS NOLIST;
  delete TEMP1 TEMP1Y TEMP_2 TEMP_3 TEMP_4;;
 run;

 %MEND;

%MOD3(21,APS21);
%MOD3(22,APS22);
%MOD3(23,APS23);
%MOD3(24,APS24);
%MOD3(25,APS25);
%MOD3(26,APS26);
%MOD3(27,APS27);


/* MERGE THE ADJUSTED DENOMINATOR AND NUMERATOR FOR AREA PATIENT SAFETY INDICATORS. */

 data TEMP2Y;
   merge ADJ_APS21 ADJ_APS22 ADJ_APS23 ADJ_APS24 
         ADJ_APS25 ADJ_APS26 ADJ_APS27;
   by MAREA AGECAT SEXCAT RACECAT;
 run;
 
  * -------------------------------------------------------------- ; 
  * --- PATIENT SAFETY INDICATOR (PSI) NAMING CONVENTION:      --- ; 
  * --- THE FIRST LETTER IDENTIFIES THE PATIENT SAFETY         --- ; 
  * --- INDICATOR AS ONE OF THE FOLLOWING: 
                  (T) NUMERATOR ("TOP") - FROM PSI_ALL_MEASURES 
                  (P) POPULATION ("POP") - FROM PSI_ALL_MEASURES 
                  (O) OBSERVED RATES 
  * --- THE SECOND LETTER IDENTIFIES THE PSI AS A PROVIDER (P) --- ; 
  * --- OR AN AREA (A) LEVEL INDICATOR.  THE                   --- ; 
  * --- NEXT TWO CHARACTERS ARE ALWAYS 'PS'. THE LAST TWO      --- ; 
  * --- DIGITS ARE THE INDICATOR NUMBER (WITHIN THAT SUBTYPE). --- ; 
  * -------------------------------------------------------------- ; 

 /* CALCULATE OBSERVED RATE AS SUM OF NUMERATOR / SUM OF ADJUSTED DENOMINATOR.*/
 data &OUTFILE_AREAOBS.;
 set TEMP2Y;

 array PPS2{7} PAPS21-PAPS27 ;
 array TPS{7}  TAPS21-TAPS27 ;
 array OPS{7}  OAPS21-OAPS27 ;

 do J = 1 to 7;
    if TPS{J} gt 0 and PPS2{J} gt 0 then OPS{J} = TPS{J} / PPS2{J};
    else if PPS2{J} gt 0 then OPS{J} = 0 ;
 end;

 label
 TAPS21 = 'PSI 21 Retained Surgical Item or Unretrieved Device Fragment Rate (Numerator)'
 TAPS22 = 'PSI 22 Iatrogenic Pneumothorax Rate (Numerator)'
 TAPS23 = 'PSI 23 Central Venous Catheter-Related Blood Stream Infection Rate (Numerator)'
 TAPS24 = 'PSI 24 Postoperative Wound Dehiscence Rate (Numerator)'
 TAPS25 = 'PSI 25 Accidental Puncture or Laceration Rate (Numerator)'
 TAPS26 = 'PSI 26 Transfusion Reaction Rate (Numerator)'
 TAPS27 = 'PSI 27 Perioperative Hemorrhage or Hematoma Rate (Numerator)'
 ;

 label
 PAPS21 = 'PSI 21 Retained Surgical Item or Unretrieved Device Fragment Rate (Denominator)'
 PAPS22 = 'PSI 22 Iatrogenic Pneumothorax Rate (Denominator)'
 PAPS23 = 'PSI 23 Central Venous Catheter-Related Blood Stream Infection Rate (Denominator)'
 PAPS24 = 'PSI 24 Postoperative Wound Dehiscence Rate (Denominator)'
 PAPS25 = 'PSI 25 Accidental Puncture or Laceration Rate (Denominator)'
 PAPS26 = 'PSI 26 Transfusion Reaction Rate (Denominator)'
 PAPS27 = 'PSI 27 Perioperative Hemorrhage or Hematoma Rate (Denominator)'
 ;

 label
 OAPS21 = 'PSI 21 Retained Surgical Item or Unretrieved Device Fragment Rate (Observed)'
 OAPS22 = 'PSI 22 Iatrogenic Pneumothorax Rate (Observed)'
 OAPS23 = 'PSI 23 Central Venous Catheter-Related Blood Stream Infection Rate (Observed)'
 OAPS24 = 'PSI 24 Postoperative Wound Dehiscence Rate (Observed)'
 OAPS25 = 'PSI 25 Accidental Puncture or Laceration Rate (Observed)'
 OAPS26 = 'PSI 26 Transfusion Reaction Rate (Observed)'
 OAPS27 = 'PSI 27 Perioperative Hemorrhage or Hematoma Rate (Observed)'
 _TYPE_ = 'STRATIFICATION LEVEL  '
 ;

 drop J;

 run;

 proc Datasets nolist;
   delete MAREA QIPOP QIPOP0 TEMP0 TEMP2Y
          ADJ_APS21 ADJ_APS22 ADJ_APS23 ADJ_APS24 
          ADJ_APS25 ADJ_APS26 ADJ_APS27;
 run; quit;

 proc Sort data=&OUTFILE_AREAOBS. out=OUTAOBS.&OUTFILE_AREAOBS.;
   by MAREA AGECAT SEXCAT RACECAT;
 run;

 * -------------------------------------------------------------- ;
 * --- CONTENTS AND MEANS OF AREA OBSERVED MEANS FILE ----------- ;
 * -------------------------------------------------------------- ;

 proc Contents data=OUTAOBS.&OUTFILE_AREAOBS. position;
 run;

***----- TO PRINT VARIABLE LABELS COMMENT (DELETE) "NOLABELS" FROM PROC MEANS STATEMENTS -------***;

 proc Means data=OUTAOBS.&OUTFILE_AREAOBS. (where=(_TYPE_ in (8))) n nmiss min max sum nolabels;
   var TAPS21-TAPS27 ;
   title  'SUMMARY OF PATIENT SAFETY AREA LEVEL INDICATOR OVERALL NUMERATOR (SUM) WHEN _TYPE_=8';
 run; quit;

 proc Means data=OUTAOBS.&OUTFILE_AREAOBS. (where=(_TYPE_ in (8))) n nmiss min max sum nolabels;
   var PAPS21-PAPS27 ;
   title  'SUMMARY OF PATIENT SAFETY AREA LEVEL INDICATOR OVERALL DENOMINATOR (SUM) WHEN _TYPE_=8';
 run; quit;

 proc Means data=OUTAOBS.&OUTFILE_AREAOBS. (where=(_TYPE_ in (8))) n nmiss min max mean nolabels;
   var OAPS21-OAPS27;
   title  'SUMMARY OF PATIENT SAFETY AREA LEVEL INDICATOR AVERAGE RATES(MEAN) WHEN _TYPE_=8';
 run; quit;


 proc Contents data=OUTAOBS.&OUTFILE_AREAOBS. position;
 run;

 * -------------------------------------------------------------- ;
 * --- PRINT AREA OBSERVED MEANS FILE --------------------------- ;
 * -------------------------------------------------------------- ;

 %MACRO PRT2;

 %IF &PRINT. = 1 %THEN %DO;

 %MACRO PRT(PS,TEXT);

 proc  Print data=OUTAOBS.&OUTFILE_AREAOBS. label split='*';
 var   MAREA AGECAT SEXCAT RACECAT TAPS&PS. PAPS&PS. OAPS&PS. ;
 label MAREA   = "MAREA"
       AGECAT  = "AGECAT"
       SEXCAT  = "SEXCAT"
       RACECAT = "RACECAT"
       TAPS&PS.  = "TAPS&PS.*(Numerator)"
       PAPS&PS.  = "PAPS&PS.*(Denominator)"
       OAPS&PS.  = "OAPS&PS.*(Observed)"
       ;
 format AGECAT AGECAT.   
        SEXCAT SEXCAT.
        RACECAT RACECAT.
        TAPS&PS. PAPS&PS. COMMA13.0
	    OAPS&PS. 8.6
        ;
 title4 "Indicator &PS.: &TEXT";
 run;

 %MEND PRT;

 %PRT(21,Retained Surgical Item or Unretrieved Device Fragment Rate);
 %PRT(22,Iatrogenic Pneumothorax Rate);
 %PRT(23,Central Venous Catheter-Related Blood Stream Infection Rate);
 %PRT(24,Postoperative Wound Dehiscence Rate);
 %PRT(25,Accidental Puncture or Laceration Rate);
 %PRT(26,Transfusion Reaction Rate);
 %PRT(27,Perioperative Hemorrhage or Hematoma Rate);

 %END;

 %MEND PRT2;

 %PRT2; 

 
 * -------------------------------------------------------------- ;
 * --- WRITE SAS OUTPUT DATA SET TO COMMA-DELIMITED TEXT FILE --- ;
 * --- FOR EXPORT INTO SPREADSHEETS                           --- ;
 * -------------------------------------------------------------- ;

 %MACRO TEXT;
 %if &TXTAOBS. = 1  %then %do;
	%LET TYPEARN  = %sysfunc(tranwrd(&TYPELVLA.,%str(,),_));
	%LET TYPEARN2  = %sysfunc(compress(&TYPEARN.,'(IN )'));
	%let PSCSVREF = %sysfunc(pathname(PSTXTAOB));
	%let PSCSVRF2 =  %sysfunc(tranwrd(&PSCSVREF.,.TXT,_&TYPEARN2..TXT));

	 data _NULL_;
 set OUTAOBS.&OUTFILE_AREAOBS.;
 file  "&PSCSVRF2." lrecl=2000 dsd;
 if _N_=1 then do;
  put "MAREA" "," "Age" "," "Sex" "," "Race" "," "Type" ","
 "TAPS21" "," "TAPS22" "," "TAPS23" "," "TAPS24" "," "TAPS25" "," "TAPS26" "," "TAPS27" ","
 "PAPS21" "," "PAPS22" "," "PAPS23" "," "PAPS24" "," "PAPS25" "," "PAPS26" "," "PAPS27" ","
 "OAPS21" "," "OAPS22" "," "OAPS23" "," "OAPS24" "," "OAPS25" "," "OAPS26" "," "OAPS27";

 end;

 
 put MAREA $5. "," AGECAT 3. "," SEXCAT 3. "," RACECAT 3. "," _TYPE_ 2. ","
 (TAPS21-TAPS27) (7.0 ",")
 ","
 (PAPS21-PAPS27) (13.2 ",")
 ","
 (OAPS21-OAPS27) (12.10 ",")
 ;
 run;

 %END;

 %MEND TEXT;

 %TEXT
