*================== Program: PSI_PROV_COMPOSITE.SAS =====================;
*
*  DESCRIPTION:
*         Constructs a Patient Safety and Adverse Events Composite 
*         measure based on user defined weights.  The default weights are the
          National Quality Forum endorsed 10 component harm adjusted
          numerator weights.
*
*                >>>  AHRQ QI ICD-09 VERSION 6.0.2  <<<
*                    >>> VERSION DATE JULY 2017 <<<
*
*  USER NOTE:  Make sure you have run PSI_PROV_RISKADJ.SAS
*              BEFORE running this program.
*
*  USER NOTE2: See the PSI_ICD09_v6_Release_Notes.txt file for 
*              software change notes.
*========================================================================;

*PATHNAME specifies the location of the PSI folder which includes the
          Programs, Macros, and SASdata subfolders;
%LET PATHNAME=C:\PATHNAME\PSI;                           *<===USER MUST modify; 
filename CONTROL "&PATHNAME.\Programs\PSI_ALL_CONTROL.SAS"; 
%INCLUDE CONTROL;

*===================================================================;
* When using a ten component index (lacking PSI-7) set DEL7 to 1.
* When using the eleven component index set DEL7 to 0.
*===================================================================;
%LET DEL7=1;

*===================================================================;
* Import PSI Composite Weights                                   --;
* Weights sum to 1.0                                        --;
*===================================================================;
%include MacLib(PSI_Composite_Wt_v60.sas);
%setweights;

*===================================================================;
*  COMPUTE COMPOSITE                                              --;
*===================================================================;

data OUTPCOMP.&OUTFILE_COMP.(keep=HOSPID PSC90 PSC90VAR PSC90SE PSC90WHT PSC90LB PSC90UB);
set OUTPRISK.&OUTFILE_PROVRISK.;

*-- Load Component Reference Rate and Signal Variance Arrays --*;
%include MacLib(&PSCOMPRR.);

array ARRY4{11} 
   RPPS03 RPPS06 RPPS07 RPPS08 RPPS09 RPPS10
   RPPS11 RPPS12 RPPS13 RPPS14 RPPS15
;

array ARRY6{11} 
   VPPS03 VPPS06 VPPS07 VPPS08 VPPS09 VPPS10 
   VPPS11 VPPS12 VPPS13 VPPS14 VPPS15
;

array ARRY7{11} 
   APPS03 APPS06 APPS07 APPS08 APPS09 APPS10 
   APPS11 APPS12 APPS13 APPS14 APPS15
;

array ARRY12{11} _temporary_
   (&W03. &W06. &W07. &W08. &W09. &W10. 
    &W11. &W12. &W13. &W14. &W15.)
;

array ARRY13{11} 
   PPPS03 PPPS06 PPPS07 PPPS08 PPPS09 PPPS10 
   PPPS11 PPPS12 PPPS13 PPPS14 PPPS15
;

do I = 1 TO 11;
   if ARRY13(I) GE 3 then do;
      ARRY6(I) = ARRY6(I) / (ARRYP3(I) * ARRYP3(I));
      do J = I TO 11;
         IDX = ARRY10(I,J);
         if I = J then ARRY7(I) = ARRY1(IDX) / (ARRY1(IDX) + ARRY6(I));
      end;
   end;
   else do;
      do J = I TO 11;
         IDX = ARRY10(I,J);
         if I = J then ARRY7(I) = 0;
      end;
   end;
end;

PSC90    = 0;
PSC90VAR = 0;
PSC90SE  = 0;
PSC90WHT = 0;

do I = 1 TO 11;
   if ARRY13(I) GE 3 then do;
      PSC90 = PSC90 + (ARRY12(I) * (((ARRY4(I) /ARRYP3(I)) * ARRY7(I)) + (1 - ARRY7(I))));
      PSC90WHT = PSC90WHT + (ARRY12(I) * ARRY13(I));
   end;
   else do;
      PSC90 = PSC90 + ARRY12(I);
      PSC90WHT = PSC90WHT + 0;
   end;
   do J = I TO 11;
      IDX = ARRY10(I,J);
      if I = J then PSC90VAR = PSC90VAR 
      + (ARRY12(I) * (ARRY1(IDX) * (1 - ARRY7(I))) * ARRY12(J));
      else PSC90VAR = PSC90VAR 
      + (ARRY12(I) * (ARRY1(IDX) * (1 - ARRY7(I)) * (1 - ARRY7(J))) * ARRY12(J));
   end;
end;

PSC90SE  = SQRT(PSC90VAR);
PSC90LB  = PSC90 - 1.96* PSC90SE;
PSC90UB  = PSC90 + 1.96* PSC90SE;



label
  PSC90    = 'PSI 90 Patient Safety and Adverse Events Composite'
  PSC90VAR = 'PSI 90 Patient Safety and Adverse Events Composite (Variance)'
  PSC90SE  = 'PSI 90 Patient Safety and Adverse Events Composite (SE)'
  PSC90WHT = 'PSI 90 Patient Safety and Adverse Events Composite (Weighted Denominator)'
  PSC90LB  = 'PSI 90 Patient Safety and Adverse Events Composite (Lower CL)'
  PSC90UB  = 'PSI 90 Patient Safety and Adverse Events Composite (Upper CL)';

run;



%MACRO PRT;

%IF &PRINT. = 1 %THEN %DO;

proc Means data=OUTPCOMP.&OUTFILE_COMP. n mean;
title 'PATIENT SAFETY AND ADVERSE EVENTS COMPOSITE';
var PSC90 PSC90VAR PSC90SE PSC90WHT;
run;

%END;

%MEND;

%PRT;
