*========================== Program: PSI_PROV_RISKADJ.SAS ==========================;
*
*  DESCRIPTION:
*        Calculates Risk-Adjusted and Smoothed Rates for Patient Safety
*        Indicators, adjusting for: SEX, AGE, DRG, MDC AND COMORBIDITY.
*        Variables created by the program: RPPSxx, LPPSxx, LPPSxx, SPPSxx, XPPSxx
*        Output stratified by HOSPID AGECAT SEXCAT PAYCAT RACECAT. 
*                
*                      >>>  AHRQ QI ICD-09 VERSION 6.0.2 <<<
*                          >>> VERSION DATE JULY 2017 <<<
*
*  USER NOTE: The AHRQ QI software does not support the calculation of weighted   
*             estimates and standard errors using complex sampling designs.  
*
*             See the PSI_ICD09_v6_Release_Notes.txt file for version change details.
*===================================================================================;

*PATHNAME specifies the location of the PSI folder which includes the
          Programs, Macros, and SASdata subfolders;
%LET PATHNAME=C:\PATHNAME\PSI;                           *<===USER MUST modify; 
filename CONTROL "&PATHNAME.\Programs\PSI_ALL_CONTROL.SAS"; 
%INCLUDE CONTROL;

 title2 'PROGRAM PSI_PROV_RISKADJ';
 title3 'AHRQ PATIENT SAFETY INDICATORS: CALCULATE RISK-ADJUSTED PROVIDER RATES';
 * ------------------------------------------------------------------ ;
* --- PATIENT SAFETY INDICATOR (PSI) NAMING CONVENTION:          --- ;
* --- THE FIRST LETTER IDENTIFIES THE PATIENT SAFETY             --- ;
* --- INDICATOR AS ONE OF THE FOLLOWING:
            (T) NUMERATOR ("TOP") - FROM PSI_ALL_MEASURES
            (P) DENOMINATOR ("POP") - FROM PSI_ALL_MEASURES
            (O) OBSERVED RATES (T/P)
            (E) EXPECTED RATE
            (V) VARIANCE 
            (R) RISK-ADJUSTED RATE
            (L) LOWER CONFIDENCE LIMIT FOR RISK-ADJUSTED RATE
            (U) UPPER CONFIDENCE LIMIT FOR RISK-ADJUSTED RATE
            (S) SMOOTHED RATE (NOT REPORTED FOR STRATA)
            (X) SMOOTHED RATE STANDARD ERROR (NOT REPORTED FOR STRATA)
* --- THE SECOND LETTER IDENTIFIES THE PSI AS A PROVIDER (P)     --- ;
* --- LEVEL INDICATOR.  THE NEXT TWO CHARACTERS ARE ALWAYS 'PS'. --- ;
* --- THE LAST TWO DIGITS ARE THE INDICATOR NUMBER               --- ;
* ------------------------------------------------------------------ ;
*---------------------------------------------------------------------* ;
*-- The PSI_Prov_Regvars_v60.sas program creates the variables      --* ;
*-- used for risk adjustment. The user must ensure the the          --* ;
*-- PSI_ALL_CONTROL program includes the correct location for the   --* ;
*-- PSI_Prov_Regvars_v60 SAS code.                                  --* ;
*---------------------------------------------------------------------* ;

data &OUTFILE_MEAS.;
    set OUTPOBS.&OUTFILE_MEAS.;
    %include Maclib(PSI_Prov_Regvars_v60.sas);
run ;

*** Change the order of SAS variables to support double-dash variable reference;

data &OUTFILE_MEAS. ;
  retain key HOSPID TPPS: 
   &agesx_ &comorb_ &mdc_ &mdrgs_
   TRNSFER ;
  set &OUTFILE_MEAS. ;
run ;


%MACRO MOD3(PS, CV);      

  data temp;
    length TYPELVLP $30. MODEL $10.;
    TYPELVLP = "&TYPELVLP.";

    AGE = 0;
    SEX = 0;
    MOD = .;
    CONFLICT = 0;
    do i = 1 to 16;
        TYPE = scan(TYPELVLP,i,', ( ) IN'); 
        if TYPE in ('12' '13' '14' '15' '28' '29' '30' '31') then do;
            AGE = 1; SEX = 1;
            if MOD not in (.,3) then CONFLICT = 1; MOD = 3; end;
        if TYPE in ('8' '9' '10' '11' '24' '25' '26' '27') then do;
            AGE = 1;
            if MOD not in (.,1) then CONFLICT = 1; MOD = 1; end;
        if TYPE in ('4' '5' '6' '7' '20' '21' '22' '23') then do;
            SEX = 1;
            if MOD not in (.,2) then CONFLICT = 1; MOD = 2; end;
        else do;
            if MOD not in (.,0) then CONFLICT = 1; MOD = 0; end;
    end;
    
    if not AGE and not SEX then MODEL = "AGE_SEX";
    else if AGE and SEX then MODEL = "NO_AGE_SEX";
    else if AGE then MODEL = "NO_AGE";
    else MODEL = "NO_SEX";

    call symput("MODEL_",trim(MODEL));
    call symput("ERROR_",put(CONFLICT,1.));
  run;

  %macro processNotes;
    %put NOTE: TYPELVLP &TYPELVLP. requested, thus &model_. GEE Model will be used.;
    %if &error_. = 1 %then %do;
        %put WARNING: Stratifications of Age and/or Sex have been mixed, be aware that the numerical results for non-Age/Sex categories will be incorrect ;
    %end;
  %mend;
  %processNotes;

  *** Start discharge scoring for all but the overall stratified measure, PSI04. ;
%if &ps. ^= 04 %then %do; 

 *-- PARAMETER FILE USED FOR SCORING --*;

 filename RACOEFFS  "&RADIR.\GEE_PS&PS._&MODEL_..csv";

 *-- LOAD CSV PARAMTERS & SHAPE DATA  --*;

 data TEMP1_MODEL ;
   length VARIABLE $20 DF ESTIMATE 8 ;
   infile RACOEFFS DSD DLM=',' LRECL=1024;
   input VARIABLE DF ESTIMATE ;
 run ;

 proc TRANSPOSE data=TEMP1_MODEL out=TEMP2_MODEL  ;
     id variable;
       var ESTIMATE ;
 run ;

 data MODEL_PS&PS.;
   set TEMP2_MODEL ;

   _NAME_ = "MHAT" ;
   _TYPE_ = "PARMS" ;
 run ;

 data _null_;
    set TEMP1_MODEL end=LAST;
    format vars $5000.;
    retain vars;

    if variable ne "Intercept" then  vars = trim(vars)||" "||trim(variable);
    if LAST then call symput("VARS_",vars);
run;

 *-- APPLY PROC SCORE TO DATA --* ;                   

 proc Score data=&OUTFILE_MEAS. (where=(TPPS&PS. in (0,1))) score=MODEL_PS&PS. type=PARMS out=TEMP1Y&ps.;
   var &VARS_. ;
 run;
%END; *** End discharge scoring for all but the overall stratified measure, PSI04. ;

%LET DSID=%SYSFUNC(OPEN(TEMP1Y&ps.));
%IF &dsid. %THEN %LET DNUM=%SYSFUNC(ATTRN(&dsid,NOBS));
%ELSE %LET DNUM=0;
%LET DRC=%SYSFUNC(CLOSE(&dsid));

 %IF &DNUM NE 0 %THEN %DO;

 *-- CALCULATE PREDICTED VALUES (EHAT) --* ;
 data   TEMP1Y&ps.;
   set    TEMP1Y&ps.;

   EHAT = EXP(MHAT)/(1 + EXP(MHAT));
   PHAT = EHAT * (1 - EHAT);
   if TPPS&PS. in (1,0) then PPPS=1;
   else PPPS=0;
run;

 /*-- SUMMARIZE BY VARIOUS CLASSES --*/

 proc Summary data=TEMP1Y&ps.;
    class HOSPID AGECAT SEXCAT PAYCAT RACECAT;
    var   TPPS&PS. EHAT PHAT;
 output out=RPPS&PS. sum(TPPS&PS. PPPS EHAT PHAT)=TPPS&PS. PPPS&PS. EHAT PHAT
           n=DENOM;
 run;

 *-- APPLY RISK ADJUSTMENT & SMOOTHING --* ;
 data RPPS&PS.(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT _TYPE_
                 EPPS&PS. RPPS&PS. LPPS&PS. UPPS&PS. SPPS&PS. XPPS&PS. VPPS&PS.
                 TPPS&PS. PPPS&PS. OPPS&PS. 
                 );
   set RPPS&PS.;

   if _TYPE_ &TYPELVLP;

 *-- LOAD SIGNAL AND NOISE ARRAYS FROM TEXT FILE --* ;
 %include MacLib(PSI_Prov_Sigvar_Array_V60.sas);

 *-- MAP MEASURE NUM TO ARRAY INDEX SUB_N --* ;
 if "&PS." = "02"  then SUB_N = 1;
 if "&PS." = "03"  then SUB_N = 2;
 if "&PS." = "04"  then SUB_N = 3;
 if "&PS." = "04A"  then SUB_N = 4;
 if "&PS." = "04B"  then SUB_N = 5;
 if "&PS." = "04C"  then SUB_N = 6;
 if "&PS." = "04D"  then SUB_N = 7;
 if "&PS." = "04E"  then SUB_N = 8;
 if "&PS." = "06"  then SUB_N = 9;
 if "&PS." = "07"  then SUB_N = 10;
 if "&PS." = "08"  then SUB_N = 11;
 if "&PS." = "09"  then SUB_N = 12;
 if "&PS." = "10"  then SUB_N = 13;
 if "&PS." = "11"  then SUB_N = 14;
 if "&PS." = "12"  then SUB_N = 15;
 if "&PS." = "13"  then SUB_N = 16;
 if "&PS." = "14"  then SUB_N = 17;
 if "&PS." = "15"  then SUB_N = 18;

 *-- T = NUMERATOR     --* ;
 *-- P = DENOMINATOR   --* ;
 *-- E = EXPECTED      --* ;
 *-- R = RISK ADJUSTED --* ; 
 *-- L = LOWER CI      --* ;
 *-- U = UPPER CI      --* ;
 *-- S = SMOOTHED      --* ;
 *-- X = SMOOTHED SE   --* ;
 *-- V = VARIANCE      --* ; 

 EPPS&PS. = EHAT / PPPS&PS.;
 THAT = TPPS&PS. / PPPS&PS.;
 OPPS&PS. = TPPS&PS. / PPPS&PS.;
 
 if _TYPE_ IN (0,16) then do;
    RPPS&PS.   = (THAT / EPPS&PS.) * ARRYP3(SUB_N);
    SE&PS.  = (ARRYP3(SUB_N) / EPPS&PS.) * (1 / PPPS&PS.) * sqrt(PHAT);
    VPPS&PS.   = SE&PS.**2;
    SN&PS.  = ARRYP2(SUB_N) / (ARRYP2(SUB_N) + VPPS&PS.);
    SPPS&PS.   = (RPPS&PS. * SN&PS.) + ((1 -  SN&PS.) * ARRYP3(SUB_N));
    XPPS&PS.   = sqrt(ARRYP2(SUB_N)- (SN&PS. * ARRYP2(SUB_N)));
 end;
 else do;
    RPPS&PS.   = (THAT / EPPS&PS.);
    SE&PS.  = (1 / EPPS&PS.) * (1 / PPPS&PS.) * sqrt(PHAT);
    VPPS&PS.   = .;
    SPPS&PS.   = .;
    XPPS&PS.   = .;
 end;
 
 LPPS&PS.   = RPPS&PS. - (1.96 * SE&PS.);
 if LPPS&PS. < 0 then LPPS&PS. = 0;
 UPPS&PS.   = RPPS&PS. + (1.96 * SE&PS.);

 run;

 %END;

 %ELSE %DO;

 data RPPS&PS.;
     length HOSPID AGECAT SEXCAT PAYCAT RACECAT 3;
    HOSPID=.; AGECAT=.; SEXCAT=.; PAYCAT=.; RACECAT=0;_TYPE_=0;
    EPPS&PS=.;RPPS&PS=.;LPPS&PS=.;UPPS&PS=.;SPPS&PS=.;XPPS&PS=.;VPPS&PS=.;
    TPPS&PS=.; PPPS&PS=.; OPPS&PS=.; 
    output;
 run;

 %END;

 proc Sort data=RPPS&PS.;
   by HOSPID AGECAT SEXCAT PAYCAT RACECAT _TYPE_ ;
 run; quit;

 proc Datasets NOLIST;
   delete TEMP1 TEMP1Y TEMP1_MODEL TEMP2_MODEL;
 run;

%MEND MOD3;

%MOD3(02,23) ;
%MOD3(03,125) ; 
%MOD3(04A,87) ;
%MOD3(04B,87) ;
%MOD3(04C,87) ;
%MOD3(04D,87) ;
%MOD3(04E,87) ;

/*-- Concatenate scored discharges for measure PSI04 --*/
/*-- This applies risk - adjustment from individual  --*/ 
/*-- strata to the overall measure                   --*/

data TEMP1Y04;
set TEMP1Y04A TEMP1Y04B TEMP1Y04C TEMP1Y04D TEMP1Y04E;
run;

%MOD3(04,87) ;
%MOD3(06,24) ;
%MOD3(07,92) ;
%MOD3(08,2) ;
%MOD3(09,72) ;
%MOD3(10,18) ;
%MOD3(11,86) ;
%MOD3(12,108) ;
%MOD3(13,36) ;
%MOD3(14,15) ;
%MOD3(15,99) ;

  
 * --- MERGE THE PROVIDER ADJUSTED RATES FOR EACH INDICATOR (PS.) --- ;

 data RISKADJ;
 merge RPPS02(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS02 RPPS02 LPPS02 UPPS02 SPPS02 XPPS02 VPPS02 TPPS02 PPPS02 OPPS02)
       RPPS03(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS03 RPPS03 LPPS03 UPPS03 SPPS03 XPPS03 VPPS03 TPPS03 PPPS03 OPPS03)

       RPPS04 (KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS04   RPPS04   LPPS04   UPPS04   SPPS04   XPPS04   VPPS04   TPPS04   PPPS04   OPPS04)
       RPPS04A(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS04A  RPPS04A  LPPS04A  UPPS04A  SPPS04A  XPPS04A  VPPS04A  TPPS04A  PPPS04A  OPPS04A)
       RPPS04B(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS04B  RPPS04B  LPPS04B  UPPS04B  SPPS04B  XPPS04B  VPPS04B  TPPS04B  PPPS04B  OPPS04B)
       RPPS04C(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS04C  RPPS04C  LPPS04C  UPPS04C  SPPS04C  XPPS04C  VPPS04C  TPPS04C  PPPS04C  OPPS04C)
       RPPS04D(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS04D  RPPS04D  LPPS04D  UPPS04D  SPPS04D  XPPS04D  VPPS04D  TPPS04D  PPPS04D  OPPS04D)
       RPPS04E(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS04E  RPPS04E  LPPS04E  UPPS04E  SPPS04E  XPPS04E  VPPS04E  TPPS04E  PPPS04E  OPPS04E)

       RPPS06(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS06 RPPS06 LPPS06 UPPS06 SPPS06 XPPS06 VPPS06 TPPS06 PPPS06 OPPS06)
       RPPS07(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS07 RPPS07 LPPS07 UPPS07 SPPS07 XPPS07 VPPS07 TPPS07 PPPS07 OPPS07)
       RPPS08(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS08 RPPS08 LPPS08 UPPS08 SPPS08 XPPS08 VPPS08 TPPS08 PPPS08 OPPS08)
       RPPS09(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS09 RPPS09 LPPS09 UPPS09 SPPS09 XPPS09 VPPS09 TPPS09 PPPS09 OPPS09)
       RPPS10(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS10 RPPS10 LPPS10 UPPS10 SPPS10 XPPS10 VPPS10 TPPS10 PPPS10 OPPS10)
       RPPS11(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS11 RPPS11 LPPS11 UPPS11 SPPS11 XPPS11 VPPS11 TPPS11 PPPS11 OPPS11)
       
       RPPS12(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS12 RPPS12 LPPS12 UPPS12 SPPS12 XPPS12 VPPS12 TPPS12 PPPS12 OPPS12)
       RPPS13(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS13 RPPS13 LPPS13 UPPS13 SPPS13 XPPS13 VPPS13 TPPS13 PPPS13 OPPS13)
       RPPS14(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS14 RPPS14 LPPS14 UPPS14 SPPS14 XPPS14 VPPS14 TPPS14 PPPS14 OPPS14)
       RPPS15(KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT EPPS15 RPPS15 LPPS15 UPPS15 SPPS15 XPPS15 VPPS15 TPPS15 PPPS15 OPPS15);
 by HOSPID AGECAT SEXCAT PAYCAT RACECAT;

label
 PPPS02 = 'PSI 02 Death Rate in Low-Mortality Diagnosis Related Groups (DRGs) (Pop)'
 PPPS03 = 'PSI 03 Pressure Ulcer Rate (Pop)'
 PPPS04 = 'PSI 04 Death Rate among Surgical Inpatients with Serious Treatable Conditions (Pop)'
 PPPS04A= 'PSI 04A Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum A: Deep Vein Thrombosis/Pulmonary Embolism (DVT/PE) (Pop)'
 PPPS04B= 'PSI 04B Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum B: Pneumonia (Pop)'
 PPPS04C= 'PSI 04C Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum C: Sepsis (Pop)'
 PPPS04D= 'PSI 04D Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum D: Shock/Cardiac Arrest (Pop)'
 PPPS04E= 'PSI 04E Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum E: Gastrointestinal (GI) Hemorrhage/Acute Ulcer (Pop)'
 PPPS06 = 'PSI 06 Iatrogenic Pneumothorax Rate (Pop)'
 PPPS07 = 'PSI 07 Central Venous Catheter-Related Blood Stream Infection Rate (Pop)'
 PPPS08 = 'PSI 08 In Hospital Fall with Hip Fracture Rate (Pop)'
 PPPS09 = 'PSI 09 Perioperative Hemorrhage or Hematoma Rate (Pop)'
 PPPS10 = 'PSI 10 Postoperative Acute Kidney Injury Requiring Dialysis Rate (Pop)'
 PPPS11 = 'PSI 11 Postoperative Respiratory Failure Rate (Pop)'
 PPPS12 = 'PSI 12 Perioperative Pulmonary Embolism or Deep Vein Thrombosis Rate (Pop)'
 PPPS13 = 'PSI 13 Postoperative Sepsis Rate (Pop)'
 PPPS14 = 'PSI 14 Postoperative Wound Dehiscence Rate (Pop)'
 PPPS15 = 'PSI 15 Unrecognized Abdominopelvic Accidental Puncture or Laceration Rate (Pop)'
 ;

 label
 OPPS02 = 'PSI 02 Death Rate in Low-Mortality Diagnosis Related Groups (DRGs) (Observed)'
 OPPS03 = 'PSI 03 Pressure Ulcer Rate (Observed)'
 OPPS04 = 'PSI 04 Death Rate among Surgical Inpatients with Serious Treatable Conditions (Observed)'
 OPPS04A= 'PSI 04A Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum A: Deep Vein Thrombosis/Pulmonary Embolism (DVT/PE) (Observed)'
 OPPS04B= 'PSI 04B Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum B: Pneumonia (Observed)'
 OPPS04C= 'PSI 04C Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum C: Sepsis (Observed)'
 OPPS04D= 'PSI 04D Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum D: Shock/Cardiac Arrest (Observed)'
 OPPS04E= 'PSI 04E Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum E: Hemorrhage/Acute Ulcer (Observed)'
 OPPS06 = 'PSI 06 Iatrogenic Pneumothorax Rate (Observed)'
 OPPS07 = 'PSI 07 Central Venous Catheter-Related Blood Stream Infection Rate (Observed)'
 OPPS08 = 'PSI 08 In Hospital Fall with Hip Fracture Rate (Observed)'
 OPPS09 = 'PSI 09 Perioperative Hemorrhage or Hematoma Rate (Observed)'
 OPPS10 = 'PSI 10 Postoperative Acute Kidney Injury Requiring Dialysis Rate (Observed)'
 OPPS11 = 'PSI 11 Postoperative Respiratory Failure Rate (Observed)'
 OPPS12 = 'PSI 12 Perioperative Pulmonary Embolism or Deep Vein Thrombosis Rate (Observed)'
 OPPS13 = 'PSI 13 Postoperative Sepsis Rate (Observed)'
 OPPS14 = 'PSI 14 Postoperative Wound Dehiscence Rate (Observed)'
 OPPS15 = 'PSI 15 Unrecognized Abdominopelvic Accidental Puncture or Laceration Rate (Observed)'
 ;

 label
 EPPS02 = 'PSI 02 Death Rate in Low-Mortality Diagnosis Related Groups (DRGs) (Expected)'
 EPPS03 = 'PSI 03 Pressure Ulcer Rate (Expected)'
 EPPS04 = 'PSI 04 Death Rate among Surgical Inpatients with Serious Treatable Conditions (Expected)'
 EPPS04A= 'PSI 04A Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum A: Deep Vein Thrombosis/Pulmonary Embolism (DVT/PE) (Expected)'
 EPPS04B= 'PSI 04B Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum B: Pneumonia (Expected)'
 EPPS04C= 'PSI 04C Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum C: Sepsis (Expected)'
 EPPS04D= 'PSI 04D Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum D: Shock/Cardiac Arrest (Expected)'
 EPPS04E= 'PSI 04E Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum E: Gastrointestinal (GI) Hemorrhage/Acute Ulcer (Expected)'
 EPPS06 = 'PSI 06 Iatrogenic Pneumothorax Rate (Expected)'
 EPPS07 = 'PSI 07 Central Venous Catheter-Related Blood Stream Infection Rate (Expected)'
 EPPS08 = 'PSI 08 In Hospital Fall with Hip Fracture Rate (Expected)'
 EPPS09 = 'PSI 09 Perioperative Hemorrhage or Hematoma Rate (Expected)'
 EPPS10 = 'PSI 10 Postoperative Acute Kidney Injury Requiring Dialysis Rate (Expected)'
 EPPS11 = 'PSI 11 Postoperative Respiratory Failure Rate (Expected)'
 EPPS12 = 'PSI 12 Perioperative Pulmonary Embolism or Deep Vein Thrombosis Rate (Expected)'
 EPPS13 = 'PSI 13 Postoperative Sepsis Rate (Expected)'
 EPPS14 = 'PSI 14 Postoperative Wound Dehiscence Rate (Expected)'
 EPPS15 = 'PSI 15 Unrecognized Abdominopelvic Accidental Puncture or Laceration Rate (Expected)'
 ;

 label
 RPPS02 = 'PSI 02 Death Rate in Low-Mortality Diagnosis Related Groups (DRGs) (Risk Adj)'
 RPPS03 = 'PSI 03 Pressure Ulcer Rate (Risk Adj)'
 RPPS04 = 'PSI 04 Death Rate among Surgical Inpatients with Serious Treatable Conditions (Risk Adj)'
 RPPS04A= 'PSI 04A Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum A: Deep Vein Thrombosis/Pulmonary Embolism (DVT/PE) (Risk Adj)'
 RPPS04B= 'PSI 04B Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum B: Pneumonia (Risk Adj)'
 RPPS04C= 'PSI 04C Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum C: Sepsis (Risk Adj)'
 RPPS04D= 'PSI 04D Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum D: Shock/Cardiac Arrest (Risk Adj)'
 RPPS04E= 'PSI 04E Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum E: Gastrointestinal (GI) Hemorrhage/Acute Ulcer (Risk Adj)'
 RPPS06 = 'PSI 06 Iatrogenic Pneumothorax Rate (Risk Adj)'
 RPPS07 = 'PSI 07 Central Venous Catheter-Related Blood Stream Infection Rate (Risk Adj)'
 RPPS08 = 'PSI 08 In Hospital Fall with Hip Fracture Rate (Risk Adj)'
 RPPS09 = 'PSI 09 Perioperative Hemorrhage or Hematoma Rate (Risk Adj)'
 RPPS10 = 'PSI 10 Postoperative Acute Kidney Injury Requiring Dialysis Rate (Risk Adj)'
 RPPS11 = 'PSI 11 Postoperative Respiratory Failure Rate (Risk Adj)'
 RPPS12 = 'PSI 12 Perioperative Pulmonary Embolism or Deep Vein Thrombosis Rate (Risk Adj)'
 RPPS13 = 'PSI 13 Postoperative Sepsis Rate (Risk Adj)'
 RPPS14 = 'PSI 14 Postoperative Wound Dehiscence Rate (Risk Adj)'
 RPPS15 = 'PSI 15 Unrecognized Abdominopelvic Accidental Puncture or Laceration Rate (Risk Adj)'
 ;

 label
 LPPS02 = 'PSI 02 Death Rate in Low-Mortality Diagnosis Related Groups (DRGs) (Lower CL)'
 LPPS03 = 'PSI 03 Pressure Ulcer Rate (Lower CL)'
 LPPS04 = 'PSI 04 Death Rate among Surgical Inpatients with Serious Treatable Conditions (Lower CL)'
 LPPS04A= 'PSI 04A Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum A: Deep Vein Thrombosis/Pulmonary Embolism (DVT/PE) (Lower CL)'
 LPPS04B= 'PSI 04B Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum B: Pneumonia (Lower CL)'
 LPPS04C= 'PSI 04C Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum C: Sepsis (Lower CL)'
 LPPS04D= 'PSI 04D Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum D: Shock/Cardiac Arrest (Lower CL)'
 LPPS04E= 'PSI 04E Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum E: Gastrointestinal (GI) Hemorrhage/Acute Ulcer (Lower CL)'
 LPPS06 = 'PSI 06 Iatrogenic Pneumothorax Rate (Lower CL)'
 LPPS07 = 'PSI 07 Central Venous Catheter-Related Blood Stream Infection Rate (Lower CL)'
 LPPS08 = 'PSI 08 In Hospital Fall with Hip Fracture Rate (Lower CL)'
 LPPS09 = 'PSI 09 Perioperative Hemorrhage or Hematoma Rate (Lower CL)'
 LPPS10 = 'PSI 10 Postoperative Acute Kidney Injury Requiring Dialysis Rate (Lower CL)'
 LPPS11 = 'PSI 11 Postoperative Respiratory Failure Rate (Lower CL)'
 LPPS12 = 'PSI 12 Perioperative Pulmonary Embolism or Deep Vein Thrombosis Rate (Lower CL)'
 LPPS13 = 'PSI 13 Postoperative Sepsis Rate (Lower CL)'
 LPPS14 = 'PSI 14 Postoperative Wound Dehiscence Rate (Lower CL)'
 LPPS15 = 'PSI 15 Unrecognized Abdominopelvic Accidental Puncture or Laceration Rate (Lower CL)'
 ;

 label
 UPPS02 = 'PSI 02 Death Rate in Low-Mortality Diagnosis Related Groups (DRGs) (Upper CL)'
 UPPS03 = 'PSI 03 Pressure Ulcer Rate (Upper CL)'
 UPPS04 = 'PSI 04 Death Rate among Surgical Inpatients with Serious Treatable Conditions (Upper CL)'
 UPPS04A= 'PSI 04A Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum A: Deep Vein Thrombosis/Pulmonary Embolism (DVT/PE) (Upper CL)'
 UPPS04B= 'PSI 04B Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum B: Pneumonia (Upper CL)'
 UPPS04C= 'PSI 04C Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum C: Sepsis (Upper CL)'
 UPPS04D= 'PSI 04D Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum D: Shock/Cardiac Arrest (Upper CL)'
 UPPS04E= 'PSI 04E Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum E: Gastrointestinal (GI) Hemorrhage/Acute Ulcer (Upper CL)'
 UPPS06 = 'PSI 06 Iatrogenic Pneumothorax Rate (Upper CL)'
 UPPS07 = 'PSI 07 Central Venous Catheter-Related Blood Stream Infection Rate (Upper CL)'
 UPPS08 = 'PSI 08 In Hospital Fall with Hip Fracture Rate (Upper CL)'
 UPPS09 = 'PSI 09 Perioperative Hemorrhage or Hematoma Rate (Upper CL)'
 UPPS10 = 'PSI 10 Postoperative Acute Kidney Injury Requiring Dialysis Rate (Upper CL)'
 UPPS11 = 'PSI 11 Postoperative Respiratory Failure Rate (Upper CL)'
 UPPS12 = 'PSI 12 Perioperative Pulmonary Embolism or Deep Vein Thrombosis Rate (Upper CL)'
 UPPS13 = 'PSI 13 Postoperative Sepsis Rate (Upper CL)'
 UPPS14 = 'PSI 14 Postoperative Wound Dehiscence Rate (Upper CL)'
 UPPS15 = 'PSI 15 Unrecognized Abdominopelvic Accidental Puncture or Laceration Rate (Upper CL)'
 ;

 label 
 SPPS02 = 'PSI 02 Death Rate in Low-Mortality Diagnosis Related Groups (DRGs) (Smoothed)'
 SPPS03 = 'PSI 03 Pressure Ulcer Rate (Smoothed)'
 SPPS04 = 'PSI 04 Death Rate among Surgical Inpatients with Serious Treatable Conditions (Smoothed)'
 SPPS06 = 'PSI 06 Iatrogenic Pneumothorax Rate (Smoothed)'
 SPPS07 = 'PSI 07 Central Venous Catheter-Related Blood Stream Infection Rate (Smoothed)'
 SPPS08 = 'PSI 08 In Hospital Fall with Hip Fracture Rate (Smoothed)'
 SPPS09 = 'PSI 09 Perioperative Hemorrhage or Hematoma Rate (Smoothed)'
 SPPS10 = 'PSI 10 Postoperative Acute Kidney Injury Requiring Dialysis Rate (Smoothed)'
 SPPS11 = 'PSI 11 Postoperative Respiratory Failure Rate (Smoothed)'
 SPPS12 = 'PSI 12 Perioperative Pulmonary Embolism or Deep Vein Thrombosis Rate (Smoothed)'
 SPPS13 = 'PSI 13 Postoperative Sepsis Rate (Smoothed)'
 SPPS14 = 'PSI 14 Postoperative Wound Dehiscence Rate (Smoothed)'
 SPPS15 = 'PSI 15 Unrecognized Abdominopelvic Accidental Puncture or Laceration Rate (Smoothed)'
 ;

 label 
 XPPS02 = 'PSI 02 Death Rate in Low-Mortality Diagnosis Related Groups (DRGs) (SMTHE SE)'
 XPPS03 = 'PSI 03 Pressure Ulcer Rate (SMTHE SE)'
 XPPS04 = 'PSI 04 Death Rate among Surgical Inpatients with Serious Treatable Conditions (SMTHE SE)'
 XPPS06 = 'PSI 06 Iatrogenic Pneumothorax Rate (SMTHE SE)'
 XPPS07 = 'PSI 07 Central Venous Catheter-Related Blood Stream Infection Rate (SMTHE SE)'
 XPPS08 = 'PSI 08 In Hospital Fall with Hip Fracture Rate (SMTHE SE)'
 XPPS09 = 'PSI 09 Perioperative Hemorrhage or Hematoma Rate (SMTHE SE)'
 XPPS10 = 'PSI 10 Postoperative Acute Kidney Injury Requiring Dialysis Rate (SMTHE SE)'
 XPPS11 = 'PSI 11 Postoperative Respiratory Failure Rate (SMTHE SE)'
 XPPS12 = 'PSI 12 Perioperative Pulmonary Embolism or Deep Vein Thrombosis Rate (SMTHE SE)'
 XPPS13 = 'PSI 13 Postoperative Sepsis Rate (SMTHE SE)'
 XPPS14 = 'PSI 14 Postoperative Wound Dehiscence Rate (SMTHE SE)'
 XPPS15 = 'PSI 15 Unrecognized Abdominopelvic Accidental Puncture or Laceration Rate (SMTHE SE)'
 ;

 label 
 VPPS02 = 'PSI 02 Death Rate in Low-Mortality Diagnosis Related Groups (DRGs) (Variance)'
 VPPS03 = 'PSI 03 Pressure Ulcer Rate (Variance)'
 VPPS04 = 'PSI 04 Death Rate among Surgical Inpatients with Serious Treatable Conditions (Variance)'
 VPPS04A= 'PSI 04A Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum A: Deep Vein Thrombosis/Pulmonary Embolism (DVT/PE) (Variance)'
 VPPS04B= 'PSI 04B Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum B: Pneumonia (Variance)'
 VPPS04C= 'PSI 04C Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum C: Sepsis (Variance)'
 VPPS04D= 'PSI 04D Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum D: Shock/Cardiac Arrest (Variance)'
 VPPS04E= 'PSI 04E Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum E: Gastrointestinal (GI) Hemorrhage/Acute Ulcer (Variance)'
 VPPS06 = 'PSI 06 Iatrogenic Pneumothorax Rate (Variance)'
 VPPS07 = 'PSI 07 Central Venous Catheter-Related Blood Stream Infection Rate (Variance)'
 VPPS08 = 'PSI 08 In Hospital Fall with Hip Fracture Rate (Variance)'
 VPPS09 = 'PSI 09 Perioperative Hemorrhage or Hematoma Rate (Variance)'
 VPPS10 = 'PSI 10 Postoperative Acute Kidney Injury Requiring Dialysis Rate (Variance)'
 VPPS11 = 'PSI 11 Postoperative Respiratory Failure Rate (Variance)'
 VPPS12 = 'PSI 12 Perioperative Pulmonary Embolism or Deep Vein Thrombosis Rate (Variance)'
 VPPS13 = 'PSI 13 Postoperative Sepsis Rate (Variance)'
 VPPS14 = 'PSI 14 Postoperative Wound Dehiscence Rate (Variance)'
 VPPS15 = 'PSI 15 Unrecognized Abdominopelvic Accidental Puncture or Laceration Rate (Variance)'
 ;

 * -------------------------------------------------------------- ;
 * --- RE-LABEL DAY DEPENDENT INDICATORS ------------------------ ;
 * -------------------------------------------------------------- ;
 %MACRO RELABEL;

 %IF &PRDAY. = 0 %THEN %DO;

 label 
 EPPS03 = 'Pressure Ulcer Rate-NO PRDAY (Expected)'
 EPPS08 = 'In Hospital Fall with Hip Fracture Rate-NO PRDAY (Expected)'
 EPPS09 = 'Perioperative Hemorrhage or Hematoma Rate-NO PRDAY (Expected)'
 EPPS10 = 'Postoperative Acute Kidney Injury Requiring Dialysis Rate-NO PRDAY (Expected)'
 EPPS11 = 'Postoperative Respiratory Failure Rate-NO PRDAY (Expected)'
 EPPS12 = 'Perioperative Pulmonary Embolism or Deep Vein Thrombosis Rate-NO PRDAY (Expected)'
 EPPS14 = 'Postoperative Wound Dehiscence Rate-NO PRDAY (Expected)'
 ;

 label 
 RPPS03 = 'Pressure Ulcer Rate-NO PRDAY (Risk Adj)'
 RPPS08 = 'In Hospital Fall with Hip Fracture Rate-NO PRDAY (Risk Adj)'
 RPPS09 = 'Perioperative Hemorrhage or Hematoma Rate-NO PRDAY (Risk Adj)'
 RPPS10 = 'Postoperative Acute Kidney Injury Requiring Dialysis Rate-NO PRDAY (Risk Adj)'
 RPPS11 = 'Postoperative Respiratory Failure Rate-NO PRDAY (Risk Adj)'
 RPPS12 = 'Perioperative Pulmonary Embolism or Deep Vein Thrombosis Rate-NO PRDAY (Risk Adj)'
 RPPS14 = 'Postoperative Wound Dehiscence Rate-NO PRDAY (Risk Adj)'
 ;

 label 
 SPPS03 = 'Pressure Ulcer Rate-NO PRDAY (Smoothed)'
 SPPS08 = 'In Hospital Fall with Hip Fracture Rate-NO PRDAY (Smoothed)'
 SPPS09 = 'Perioperative Hemorrhage or Hematoma Rate-NO PRDAY (Smoothed)'
 SPPS10 = 'Postoperative Acute Kidney Injury Requiring Dialysis Rate-NO PRDAY (Smoothed)'
 SPPS11 = 'Postoperative Respiratory Failure Rate-NO PRDAY (Smoothed)'
 SPPS12 = 'Perioperative Pulmonary Embolism or Deep Vein Thrombosis Rate-NO PRDAY (Smoothed)'
 SPPS14 = 'Postoperative Wound Dehiscence Rate-NO PRDAY (Smoothed)'
 ;

 %END;
 %MEND RELABEL;
 %RELABEL;

 run;

*==================================================================;
*  Merge provider observed rates for AHRQ patient safety indicators
*===================================================================;

title2 'PROGRAM PSI_PROV_RISKADJ.SAS';
title3 'AHRQ PATIENT SAFETY INDICATORS: PROVIDER-LEVEL MERGED FILES';

* ---------------------------------------------------------------- ;
* --- PATIENT SAFETY INDICATOR MERGED RATES                      - ;
* ---------------------------------------------------------------- ;

data OUTPRISK.&OUTFILE_PROVRISK. (drop=SPPS04A SPPS04B SPPS04C SPPS04D SPPS04E XPPS04A XPPS04B XPPS04C XPPS04D XPPS04E);* drop all stratified smoothed and smoothed error variables;
merge OUTPOBS.&OUTFILE_PROVOBS.   
      RISKADJ
   (KEEP=HOSPID AGECAT SEXCAT PAYCAT RACECAT 
         EPPS02 EPPS03 EPPS04 EPPS04A EPPS04B EPPS04C EPPS04D EPPS04E EPPS06-EPPS15 
         RPPS02 RPPS03 RPPS04 RPPS04A RPPS04B RPPS04C RPPS04D RPPS04E RPPS06-RPPS15  
         LPPS02 LPPS03 LPPS04 LPPS04A LPPS04B LPPS04C LPPS04D LPPS04E LPPS06-LPPS15  
         UPPS02 UPPS03 UPPS04 UPPS04A UPPS04B UPPS04C UPPS04D UPPS04E UPPS06-UPPS15  
         SPPS02 SPPS03 SPPS04 SPPS04A SPPS04B SPPS04C SPPS04D SPPS04E SPPS06-SPPS15  
         XPPS02 XPPS03 XPPS04 XPPS04A XPPS04B XPPS04C XPPS04D XPPS04E XPPS06-XPPS15  
         VPPS02 VPPS03 VPPS04 VPPS04A VPPS04B VPPS04C VPPS04D VPPS04E VPPS06-VPPS15 
         TPPS02 TPPS03 TPPS04 TPPS04A TPPS04B TPPS04C TPPS04D TPPS04E TPPS06-TPPS15 
         PPPS02 PPPS03 PPPS04 PPPS04A PPPS04B PPPS04C PPPS04D PPPS04E PPPS06-PPPS15 
         OPPS02 OPPS03 OPPS04 OPPS04A OPPS04B OPPS04C OPPS04D OPPS04E OPPS06-OPPS15 );
 by HOSPID AGECAT SEXCAT PAYCAT RACECAT;

 array ARRY1{18} EPPS02-EPPS04 EPPS04A EPPS04B EPPS04C EPPS04D EPPS04E EPPS06-EPPS15 ;
 array ARRY2{18} RPPS02-RPPS04 RPPS04A RPPS04B RPPS04C RPPS04D RPPS04E RPPS06-RPPS15 ;
 array ARRY3{18} LPPS02-LPPS04 LPPS04A LPPS04B LPPS04C LPPS04D LPPS04E LPPS06-LPPS15 ;
 array ARRY4{18} UPPS02-UPPS04 UPPS04A UPPS04B UPPS04C UPPS04D UPPS04E UPPS06-UPPS15 ;
 array ARRY5{18} SPPS02-SPPS04 SPPS04A SPPS04B SPPS04C SPPS04D SPPS04E SPPS06-SPPS15 ;
 array ARRY6{18} XPPS02-XPPS04 XPPS04A XPPS04B XPPS04C XPPS04D XPPS04E XPPS06-XPPS15 ;
 array ARRY7{18} VPPS02-VPPS04 VPPS04A VPPS04B VPPS04C VPPS04D VPPS04E VPPS06-VPPS15 ;
 array ARRY8{18} PPPS02-PPPS04 PPPS04A PPPS04B PPPS04C PPPS04D PPPS04E PPPS06-PPPS15 ;

 do I = 1 TO 18;
    if ARRY8(I) <= 2 then do;
       ARRY1(I) = .; 
       ARRY2(I) = .; 
       ARRY3(I) = .; 
       ARRY4(I) = .;
       ARRY5(I) = .; 
       ARRY6(I) = .; 
       ARRY7(I) = .; 
    end;
 end;

 drop I;

 format EPPS02 EPPS03 EPPS04 EPPS04A EPPS04B EPPS04C EPPS04D EPPS04E EPPS06 EPPS07 EPPS08 EPPS09 EPPS10 EPPS11 
        EPPS12 EPPS13 EPPS14 EPPS15  

        LPPS02 LPPS03 LPPS04 LPPS04A LPPS04B LPPS04C LPPS04D LPPS04E LPPS06 LPPS07 LPPS08 LPPS09 LPPS10 LPPS11 
        LPPS12 LPPS13 LPPS14 LPPS15  

        OPPS02 OPPS03 OPPS04 OPPS04A OPPS04B OPPS04C OPPS04D OPPS04E OPPS06 OPPS07 OPPS08 OPPS09 OPPS10 OPPS11 
        OPPS12 OPPS13 OPPS14 OPPS15  

        RPPS02 RPPS03 RPPS04 RPPS04A RPPS04B RPPS04C RPPS04D RPPS04E RPPS06 RPPS07 RPPS08 RPPS09 RPPS10 RPPS11
        RPPS12 RPPS13 RPPS14 RPPS15      

        SPPS02 SPPS03 SPPS04 SPPS04A SPPS04B SPPS04C SPPS04D SPPS04E SPPS06 SPPS07 SPPS08 SPPS09 SPPS10 SPPS11
        SPPS12 SPPS13 SPPS14 SPPS15  

        UPPS02 UPPS03 UPPS04 UPPS04A UPPS04B UPPS04C UPPS04D UPPS04E UPPS06 UPPS07 UPPS08 UPPS09 UPPS10 UPPS11
        UPPS12 UPPS13 UPPS14 UPPS15  

        VPPS02 VPPS03 VPPS04 VPPS04A VPPS04B VPPS04C VPPS04D VPPS04E VPPS06 VPPS07 VPPS08 VPPS09 VPPS10 VPPS11
        VPPS12 VPPS13 VPPS14 VPPS15  

        XPPS02 XPPS03 XPPS04 XPPS04A XPPS04B XPPS04C XPPS04D XPPS04E XPPS06 XPPS07 XPPS08 XPPS09 XPPS10 XPPS11
        XPPS12 XPPS13 XPPS14 XPPS15   13.7

        TPPS02 TPPS03 TPPS04 TPPS04A TPPS04B TPPS04C TPPS04D TPPS04E TPPS05 TPPS06 TPPS07 TPPS08 TPPS09 TPPS10
        TPPS11 TPPS12 TPPS13 TPPS14  TPPS15  TPPS16  

        PPPS02 PPPS03 PPPS04 PPPS04A PPPS04B PPPS04C PPPS04D PPPS04E PPPS06 PPPS07 PPPS08 PPPS09 PPPS10 PPPS11 
        PPPS12 PPPS13 PPPS14 PPPS15   13.0;
 run;

* -------------------------------------------------------------- ;
* --- CONTENTS AND MEANS OF PROVIDER-LEVEL MERGED FILE --------- ;
* -------------------------------------------------------------- ;

proc Contents data=OUTPRISK.&OUTFILE_PROVRISK. position;
run; quit;

***----- TO PRINT VARIABLE LABELS COMMENT (DELETE) "NOLABELS" FROM PROC MEANS STATEMENTS -------***;

proc Means data=OUTPRISK.&OUTFILE_PROVRISK. (WHERE=(_TYPE_ IN (16))) n nmiss min max mean nolabels;
   var TPPS02 TPPS03 TPPS04 TPPS04A TPPS04B TPPS04C TPPS04D TPPS04E TPPS05-TPPS16 ;
   title  'SUMMARY OF PATIENT SAFETY QUALITY PROVIDER-LEVEL INDICATOR OVERALL NUMERATOR (SUM) WHEN _TYPE_=16';
run; quit;

proc Means data=OUTPRISK.&OUTFILE_PROVRISK. (WHERE=(_TYPE_ IN (16))) n nmiss min max mean nolabels;
   var PPPS03 PPPS04 PPPS04A PPPS04B PPPS04C PPPS04D PPPS04E PPPS06-PPPS15 ;
   title  'SUMMARY OF PATIENT SAFETY PROVIDER-LEVEL INDICATOR OVERALL DENOMINATOR (SUM) WHEN _TYPE_=16';
run; quit;

proc Means data=OUTPRISK.&OUTFILE_PROVRISK. (WHERE=(_TYPE_ IN (16))) n nmiss min max mean nolabels;
   var OPPS02 OPPS03 OPPS04 OPPS04A OPPS04B OPPS04C OPPS04D OPPS04E OPPS06-OPPS15 
       EPPS02 EPPS03 EPPS04 EPPS04A EPPS04B EPPS04C EPPS04D EPPS04E EPPS06-EPPS15  
       RPPS02 RPPS03 RPPS04 RPPS04A RPPS04B RPPS04C RPPS04D RPPS04E RPPS06-RPPS15  
       SPPS02 SPPS03 SPPS04 SPPS06-SPPS15  ;
   title  'SUMMARY OF PATIENT SAFETY PROVIDER-LEVEL INDICAOTOR AVERAGE RATES(MEAN) WHEN _TYPE_=16';
run; quit;

* -------------------------------------------------------------- ;
* --- PRINT PROVIDER MERGED FILE ------------------------------- ;
* -------------------------------------------------------------- ;

%MACRO PRT2;

%IF &PRINT. = 1 %THEN %DO;

%MACRO PRT(PS,TEXT,VOLUME,STRATA);

proc PRINT data=OUTPRISK.&OUTFILE_PROVRISK. label SPLIT='*';
%IF   &VOLUME = 0 AND &STRATA=0 %THEN %DO;
var HOSPID AGECAT SEXCAT PAYCAT RACECAT TPPS&PS. PPPS&PS. OPPS&PS. EPPS&PS. RPPS&PS. LPPS&PS. UPPS&PS. SPPS&PS. XPPS&PS.;
label HOSPID  = "HOSPID"
      AGECAT  = "AGECAT"
      SEXCAT  = "SEXCAT"
      PAYCAT  = "PAYCAT"
      RACECAT = "RACECAT"
      TPPS&PS. = "TPPS&PS.*(NUMERATOR)"
      PPPS&PS. = "PPPS&PS.*(DENOMINATOR)"
      OPPS&PS. = "OPPS&PS.*(OBSERVED)"
      EPPS&PS. = "EPPS&PS.*(EXPECTED)"
      RPPS&PS. = "RPPS&PS.*(RISK ADJ)"
      LPPS&PS. = "LPPS&PS.*(LOWER CL)"
      UPPS&PS. = "UPPS&PS.*(UPPER CL)"
      SPPS&PS. = "SPPS&PS.*(SMOOTHED)"
      XPPS&PS. = "XPPS&PS.*(SMTHE SE)"
      ;
format AGECAT AGECAT.
       SEXCAT SEXCAT.
       PAYCAT PAYCAT.
       RACECAT RACECAT.
       TPPS&PS. PPPS&PS. 13.0 OPPS&PS. RPPS&PS. EPPS&PS. SPPS&PS. XPPS&PS. 8.6;

%END;
%ELSE %IF &VOLUME. = 1 %THEN %DO;
var HOSPID AGECAT SEXCAT PAYCAT RACECAT 
    TPPS&PS. ;
label HOSPID  = "HOSPID"
      AGECAT  = "AGECAT"
      SEXCAT  = "SEXCAT"
      PAYCAT  = "PAYCAT"
      RACECAT = "RACECAT"
      TPPS&PS. = "TPPS&PS.*(NUMERATOR)"
     
      ;
format AGECAT AGECAT.
       SEXCAT SEXCAT.
       PAYCAT PAYCAT.
       RACECAT RACECAT.
       TPPS&PS. 13.0 ;
%END;
%ELSE%IF &STRATA. = 1 %THEN %DO;
var HOSPID AGECAT SEXCAT PAYCAT RACECAT TPPS&PS. PPPS&PS. OPPS&PS. EPPS&PS. RPPS&PS. LPPS&PS. UPPS&PS.;
label HOSPID  = "HOSPID"
      AGECAT  = "AGECAT"
      SEXCAT  = "SEXCAT"
      PAYCAT  = "PAYCAT"
      RACECAT = "RACECAT"
      TPPS&PS. = "TPPS&PS.*(NUMERATOR)"
      PPPS&PS. = "PPPS&PS.*(DENOMINATOR)"
      OPPS&PS. = "OPPS&PS.*(OBSERVED)"
      EPPS&PS. = "EPPS&PS.*(EXPECTED)"
      RPPS&PS. = "RPPS&PS.*(RISK ADJ)"
      LPPS&PS. = "LPPS&PS.*(LOWER CL)"
      UPPS&PS. = "UPPS&PS.*(UPPER CL)"
      ;
format AGECAT AGECAT.
       SEXCAT SEXCAT.
       PAYCAT PAYCAT.
       RACECAT RACECAT.
       TPPS&PS. PPPS&PS. 13.0 OPPS&PS. RPPS&PS. EPPS&PS. 8.6;
%END;
title4 "FINAL OUTPUT";
title5 "INDICATOR &PS.: &TEXT";

run;

%MEND PRT;

%PRT(02,Death Rate in Low-Mortality Diagnosis Related Groups (DRGs),0,0);
%PRT(03,Pressure Ulcer Rate,0,0);
%PRT(04,Death Rate among Surgical Inpatients with Serious Treatable Conditions,0,0);
%PRT(04A,Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum A: Deep Vein Thrombosis/Pulmonary Embolism (DVT/PE),0,1);
%PRT(04B,Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum B: Pneumonia,0,1);
%PRT(04C,Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum C: Sepsis,0,1);
%PRT(04D,Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum D: Shock/Cardiac Arrest,0,1);
%PRT(04E,Death Rate among Surgical Inpatients with Serious Treatable Conditions Stratum E: Gastrointestinal (GI) Hemorrhage/Acute Ulcer,0,1);
%PRT(05,Retained Surgical Item or Unretrieved Device Fragment Count,1,0);
%PRT(06,Iatrogenic Pneumothorax Rate,0,0);
%PRT(07,Central Venous Catheter-Related Blood Stream Infection Rate,0,0);
%PRT(08,In Hospital Fall with Hip Fracture Rate,0,0);
%PRT(09,Perioperative Hemorrhage or Hematoma Rate,0,0);
%PRT(10,Postoperative Acute Kidney Injury Requiring Dialysis Rate,0,0);
%PRT(11,Postoperative Respiratory Failure Rate,0,0);
%PRT(12,Perioperative Pulmonary Embolism or Deep Vein Thrombosis Rate,0,0);
%PRT(13,Postoperative Sepsis Rate,0,0);
%PRT(14,Postoperative Wound Dehiscence Rate,0,0);
%PRT(15,Unrecognized Abdominopelvic Accidental Puncture or Laceration Rate,0,0);
%PRT(16,Transfusion Reaction Count,1,0);
%END;

%MEND PRT2;

%PRT2;

* -------------------------------------------------------------- ;
* --- WRITE SAS OUTPUT DATA SET TO COMMA-DELIMITED TEXT FILE --- ;
* --- FOR EXPORT INTO SPREADSHEETS ----------------------------- ;
* -------------------------------------------------------------- ;

%MACRO TEXT;

%IF &TXTPRISK. = 1 %THEN %DO;
	%LET TYPEPRN  = %sysfunc(tranwrd(&TYPELVLP.,%str(,),_));
	%LET TYPEPRN2  = %sysfunc(compress(&TYPEPRN.,'(IN )'));
	%let PSCSVREF = %sysfunc(pathname(PSTXTPRA));
	%let PSCSVRF2 =  %sysfunc(tranwrd(&PSCSVREF.,.TXT,_&TYPEPRN2..TXT));
	 data _NULL_;
set OUTPRISK.&OUTFILE_PROVRISK.;
 file  "&PSCSVRF2." lrecl=4500 ;
 if _N_=1 then do;
  PUT
"HOSP ID" "," "AGE" "," "SEX" "," "PAYER" "," "RACE" "," "TYPE" ","
"TPPS02" "," "TPPS03" "," "TPPS04" "," "TPPS04A" "," "TPPS04B" "," "TPPS04C" "," "TPPS04D" "," "TPPS04E" ","
"TPPS05" "," "TPPS06" "," "TPPS07" "," "TPPS08"  "," "TPPS09"  "," "TPPS10"  "," "TPPS11"  "," "TPPS12" ","
"TPPS13" "," "TPPS14" "," "TPPS15" "," "TPPS16"  "," 

"PPPS02" "," "PPPS03" "," "PPPS04" "," "PPPS04A" "," "PPPS04B" "," "PPPS04C" "," "PPPS04D" "," "PPPS04E" ","
"PPPS06" "," "PPPS07" "," "PPPS08" "," "PPPS09"  "," "PPPS10"  "," "PPPS11"  "," "PPPS12"  ","
"PPPS13" "," "PPPS14" "," "PPPS15" "," 

"OPPS02" "," "OPPS03" "," "OPPS04" "," "OPPS04A" "," "OPPS04B" "," "OPPS04C" "," "OPPS04D" "," "OPPS04E" ","
"OPPS06" "," "OPPS07" "," "OPPS08" "," "OPPS09"  "," "OPPS10"  "," "OPPS11"  "," "OPPS12"  ","
"OPPS13" "," "OPPS14" "," "OPPS15" ","  

"EPPS02" "," "EPPS03" "," "EPPS04" "," "EPPS04A" "," "EPPS04B" "," "EPPS04C" "," "EPPS04D" "," "EPPS04E" "," 
"EPPS06" "," "EPPS07" "," "EPPS08" "," "EPPS09"  "," "EPPS10"  "," "EPPS11"  "," "EPPS12"  ","  
"EPPS13" "," "EPPS14" "," "EPPS15" "," 

"RPPS02" "," "RPPS03" "," "RPPS04" "," "RPPS04A" "," "RPPS04B" "," "RPPS04C" "," "RPPS04D" "," "RPPS04E" "," "RPPS06" ","
"RPPS07" "," "RPPS08" "," "RPPS09" "," "RPPS10"  "," "RPPS11"  "," "RPPS12"  "," "RPPS13"  "," "RPPS14"  ","
"RPPS15" "," 

"LPPS02" "," "LPPS03" "," "LPPS04" "," "LPPS04A" "," "LPPS04B" "," "LPPS04C" "," "LPPS04D" "," "LPPS04E" "," "LPPS06" "," 
"LPPS07" "," "LPPS08" "," "LPPS09" "," "LPPS10"  "," "LPPS11"  "," "LPPS12"  "," "LPPS13"  "," "LPPS14"  ","
"LPPS15" "," 

"UPPS02" "," "UPPS03" "," "UPPS04" "," "UPPS04A" "," "UPPS04B" "," "UPPS04C" "," "UPPS04D" "," "UPPS04E" "," "UPPS06" ","
"UPPS07" "," "UPPS08" "," "UPPS09" "," "UPPS10"  "," "UPPS11"  "," "UPPS12"  "," "UPPS13"  "," "UPPS14"  ","
"UPPS15" "," 

"SPPS02" "," "SPPS03" "," "SPPS04" "," "SPPS06" ","  
"SPPS07" "," "SPPS08" "," "SPPS09" "," "SPPS10"  "," "SPPS11"  "," "SPPS12"  "," "SPPS13"  "," "SPPS14"  ","
"SPPS15" ","  

"XPPS02" "," "XPPS03" "," "XPPS04" "," "XPPS06" ","
"XPPS07" "," "XPPS08" "," "XPPS09" "," "XPPS10"  "," "XPPS11"  "," "XPPS12"  "," "XPPS13"  "," "XPPS14"  ","
"XPPS15" ","  
;
end;

PUT HOSPID 13. "," AGECAT 3. "," SEXCAT 3. "," PAYCAT 3. "," RACECAT 3. "," _TYPE_ 2. ","
(TPPS02 TPPS03 TPPS04 TPPS04A TPPS04B TPPS04C TPPS04D TPPS04E TPPS05-TPPS16 ) (7.0  ",")  "," 
(PPPS02 PPPS03 PPPS04 PPPS04A PPPS04B PPPS04C PPPS04D PPPS04E PPPS06-PPPS15 ) (13.2 ",")  ","
(OPPS02 OPPS03 OPPS04 OPPS04A OPPS04B OPPS04C OPPS04D OPPS04E OPPS06-OPPS15 ) (12.10 ",") ","
(EPPS02 EPPS03 EPPS04 EPPS04A EPPS04B EPPS04C EPPS04D EPPS04E EPPS06-EPPS15 ) (12.10 ",") ","
(RPPS02 RPPS03 RPPS04 RPPS04A RPPS04B RPPS04C RPPS04D RPPS04E RPPS06-RPPS15 ) (12.10 ",") ","
(LPPS02 LPPS03 LPPS04 LPPS04A LPPS04B LPPS04C LPPS04D LPPS04E LPPS06-LPPS15 ) (12.10 ",") ","
(UPPS02 UPPS03 UPPS04 UPPS04A UPPS04B UPPS04C UPPS04D UPPS04E UPPS06-UPPS15 ) (12.10 ",") ","
(SPPS02 SPPS03 SPPS04 SPPS06-SPPS15 ) (12.10 ",") ","
(XPPS02 XPPS03 XPPS04 XPPS06-XPPS15 ) (12.10 ",")
;

run;

%END;

%MEND TEXT;

%TEXT;
